require "thor"
require "thor/group"
require "httparty"
require "tmpdir"

module RailsNew
  class CLI < Thor::Group
    desc "run `rails new` with the requirements of the template"

    argument :template_key,
      :type => :string,
      :desc => "The template key in the format user/template",
      :required => true

    argument :app_name,
      :type => :string,
      :desc => "The application name to pass to Rails",
      :required => true

    class_option :version, :type => :string, :required => false

    def request_template_data
      url = RailsNew::Api.template_url_for(template_key)

      say "Fetching template data from ", :green, false
      say url, :blue

      @response = HTTParty.get(url)
      if @response.code != 200
        raise "The response of the server to #{@response.request.uri} was not 200."
      end
    end

    def parse_template_data
      @template_data = JSON.parse(@response.body)['template']
      @git_repository = @template_data['git_repository']
      @git_branch = @template_data['git_branch']
      @main_file_location = @template_data["main_file_location"]
      @arguments = build_arguments_array(@template_data['arguments'])
    end

    def run_template
      Dir.mktmpdir do |temp_dir|
        template_location = Pathname.new(temp_dir).join(@main_file_location).to_s

        say "Cloning git repository ", :green, false
        say @git_repository, :blue
        clone_command = RailsNew::Command.new("git", "clone", "-b", @git_branch, @git_repository, temp_dir)
        clone_command.run

        rails_command = RailsNew::Command.new(
          "rails", "new", @app_name, "-m", template_location, *@arguments, *extra_rails_arguments
        )
        
        say "The following command will be executed:", :green, false

        puts "\n\n"
        say rails_command.to_s, :blue
        puts

        if yes? "Is this command safe?"
          say "Executing template", :green
          rails_command.run
        else
          say "Aborted command. Please review the template #{template_key}.", :red
          exit(1)
        end
      end
    end

    def exit_on_failure?
      true
    end

    private

    def build_arguments_array(arguments)
      arguments.map do |arg|
        tokens = [arg['key']]
        tokens << [arg['value']] if arg['has_value']
        tokens
      end.flatten
    end

    def extra_rails_arguments
      @extra_rails_arguments ||= ARGV[2..-1]
    end
  end
end
